<?php

namespace Drupal\disable_route_normalizer\Routing;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\Core\Routing\RoutingEvents;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;


/**
 * Listens to route events.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * The entity bundles.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityBundles;

  /**
   * The global message configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $globalConfig;

  /**
   * Constructs a new Route Subscriber.
   *
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_bundles
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The cache tags invalidator.
   */
  public function __construct(EntityTypeBundleInfoInterface $entity_bundles, ConfigFactoryInterface $config_factory) {
    $this->entityBundles = $entity_bundles;
    $this->globalConfig = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Run after EntityRouteAlterSubscriber.
    $events[RoutingEvents::ALTER][] = ['onAlterRoutes', -200];
    return $events;
  }

  /**
   * {@inheritdoc}
   */
  public function alterRoutes(RouteCollection $collection) {
    $bundles = $this->entityBundles->getBundleInfo('node');
    foreach ($bundles as $bundle_id => $bundle) {
      $bundle_config = $this->globalConfig->get("language.content_settings.node.$bundle_id");
      if ($bundle_config->get('default_langcode') == LanguageInterface::LANGCODE_NOT_SPECIFIED && !$bundle_config->get('language_alterable')) {
        $route_name = "disable_route_normalizer.node_view.$bundle_id";
        /** @var \Symfony\Component\Routing\Route $existing_route */
        $existing_route = $collection->get('entity.node.canonical');
        $path = $existing_route->getPath();
        $options = $existing_route->getOptions();
        $options['_route_filters'][] = 'disable_route_normalizer.neutral_node_filter';
        $route = new Route(
          $path,
          $existing_route->getDefaults() + [
            '_disable_route_normalizer' => TRUE,
            'disable_route_normalizer.node_view' => $bundle_id,
          ],
          $existing_route->getRequirements(),
          $options
        );

        $collection->add($route_name, $route);
      }
    }
  }
}
